package com.sun.org.apache.xalan.internal.xsltc.cmdline.getopt;

import com.sun.org.apache.xalan.internal.xsltc.compiler.util.ErrorMsg;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

public class GetOpt
{
    private GetOpt.Option theCurrentOption;
    private ListIterator theOptionsIterator;
    private List theOptions;
    private List theCmdArgs;
    private GetOpt.OptionMatcher theOptionMatcher;
    
    public GetOpt(String[] args, String optString) {
        this.theCurrentOption = null;
        this.theOptions = null;
        this.theCmdArgs = null;
        this.theOptionMatcher = null;
        this.theOptions = new ArrayList();
        int currOptIndex = 0;
        this.theCmdArgs = new ArrayList();
        this.theOptionMatcher = new GetOpt.OptionMatcher(optString);
        int i = 0;
        while (i < args.length) {
            String token = args[i];
            int tokenLength = token.length();
            if (token.equals("--")) {
                currOptIndex = i + 1;
                break;
            }
            if (token.startsWith("-") && tokenLength == 2) {
                this.theOptions.add(new GetOpt.Option(token.charAt(1)));
            }
            else if (token.startsWith("-") && tokenLength > 2) {
                for (int j = 1; j < tokenLength; ++j) {
                    this.theOptions.add(new GetOpt.Option(token.charAt(j)));
                }
            }
            else if (!token.startsWith("-")) {
                if (this.theOptions.size() == 0) {
                    currOptIndex = i;
                    break;
                }
                int indexoflast = 0;
                indexoflast = this.theOptions.size() - 1;
                GetOpt.Option op = (GetOpt.Option)this.theOptions.get(indexoflast);
                char opLetter = op.getArgLetter();
                if (!op.hasArg() && this.theOptionMatcher.hasArg(opLetter)) {
                    op.setArg(token);
                }
                else {
                    currOptIndex = i;
                    break;
                }
            }
            ++i;
        }
        this.theOptionsIterator = this.theOptions.listIterator();
        for (i = currOptIndex; i < args.length; ++i) {
            String token = args[i];
            this.theCmdArgs.add(token);
        }
    }
    
    public void printOptions() {
        ListIterator<Object> it = this.theOptions.listIterator();
        while (it.hasNext()) {
            GetOpt.Option opt = (GetOpt.Option)it.next();
            System.out.print("OPT =" + opt.getArgLetter());
            String arg = opt.getArgument();
            if (arg != null) {
                System.out.print(" " + arg);
            }
            System.out.println();
        }
    }
    
    public int getNextOption() throws IllegalArgumentException, MissingOptArgException {
        int retval = -1;
        if (this.theOptionsIterator.hasNext()) {
            this.theCurrentOption = (GetOpt.Option)this.theOptionsIterator.next();
            char c = this.theCurrentOption.getArgLetter();
            boolean shouldHaveArg = this.theOptionMatcher.hasArg(c);
            String arg = this.theCurrentOption.getArgument();
            if (!this.theOptionMatcher.match(c)) {
                ErrorMsg msg = new ErrorMsg("ILLEGAL_CMDLINE_OPTION_ERR", new Character(c));
                throw new IllegalArgumentException(msg.toString());
            }
            if (shouldHaveArg && arg == null) {
                ErrorMsg msg = new ErrorMsg("CMDLINE_OPT_MISSING_ARG_ERR", new Character(c));
                throw new MissingOptArgException(msg.toString());
            }
            retval = c;
        }
        return retval;
    }
    
    public String getOptionArg() {
        String retval = null;
        String tmp = this.theCurrentOption.getArgument();
        char c = this.theCurrentOption.getArgLetter();
        if (this.theOptionMatcher.hasArg(c)) {
            retval = tmp;
            return retval;
        }
        return retval;
    }
    
    public String[] getCmdArgs() {
        String[] retval = new String[this.theCmdArgs.size()];
        int i = 0;
        ListIterator<Object> it = this.theCmdArgs.listIterator();
        while (it.hasNext()) {
            retval[i++] = (String)it.next();
        }
        return retval;
    }
    
    class Option
    {
        private char theArgLetter;
        private String theArgument;
        
        public Option(char argLetter) {
            this.theArgument = null;
            this.theArgLetter = argLetter;
        }
        
        public void setArg(String arg) {
            this.theArgument = arg;
        }
        
        public boolean hasArg() {
            return this.theArgument != null;
        }
        
        public char getArgLetter() {
            return this.theArgLetter;
        }
        
        public String getArgument() {
            return this.theArgument;
        }
    }
    
    class OptionMatcher
    {
        private String theOptString;
        
        public OptionMatcher(String optString) {
            this.theOptString = null;
            this.theOptString = optString;
        }
        
        public boolean match(char c) {
            boolean retval = false;
            if (this.theOptString.indexOf((int)c) != -1) {
                retval = true;
            }
            return retval;
        }
        
        public boolean hasArg(char c) {
            boolean retval = false;
            int index = this.theOptString.indexOf((int)c) + 1;
            if (index == this.theOptString.length()) {
                retval = false;
            }
            else if (this.theOptString.charAt(index) == ':') {
                retval = true;
            }
            return retval;
        }
    }
}
